
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 3x, 4x, 5x
 */

#include "squidwall.h"
#include "md5.h"
#include "sha1.h"
#include "hextab.h"

/**
 * check_url - general url filtering function
 *
 * @param r    holds request specific data
 * @param dbg  can be [30|40|50] -> will be added to the debug() calls
 * @return ACL_ALLOW | ACL_DENY
 */
int check_url(request_t *r, int dbg, url_filter_t *f)
{
	struct squidwall_string *string;
	struct squidwall_regex *regex;
	struct named_regex *nr;
	struct list_head *l;
	unsigned int j, k;
	char *file, *str;

	/* ufilter */
	if (f->ufilter)
	for (j=0; j < f->ufilter->len; j += str_len(f->ufilter->s+j)+1)
	list_for_each(l, &wfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = f->ufilter->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(r->squid_url, str)) {
				debug(dbg, 5, "check_url: ACL_ALLOW @ ufilter");
				return ACL_ALLOW;
			}
		}
	}
	
	/* ufilter_url */

	/* ufilter_dom */

	/* ufilter_md5 */

	/* ufilter_sha1 */

	/* urfilter */
	if (f->urfilter)
	for (j=0; j < f->urfilter->len; j += str_len(f->urfilter->s+j)+1)
	list_for_each(l, &wrfilter) {
		file = f->urfilter->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(regex_t)); k++) {
			nr = array_get(&regex->regex, sizeof(regex_t), k);
			if (regexec(&nr->r, r->squid_url, 0, 0, nr->eflags) == 0) {
				debug(dbg, 5, "check_url: ACL_ALLOW @ urfilter");
				return ACL_ALLOW;
			}
		}
	}

	debug(dbg, 5, "check_url: ACL_ALLOW @ default");
	return ACL_ALLOW;
}
