
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 30 (general url filtering)
 */

#include "squidwall.h"
#include "md5.h"
#include "sha1.h"
#include "hextab.h"

/**
 * check_url_init - initialize url filtering
 * - initialize the two hashes if needed
 * - look for double entries in the url filters -> remove them ??
 *
 * @param r  holds request specific data
 * @return   nothing
 */
void check_url_init(request_t *r)
{
	unsigned char digest[20];
	SHA1_CTX sha1;
	MD5_CTX md5;
	char *e;
	int i,j;

	i = 0;
	if (r->uf_user.ufilter_md5) i++;
	if (r->uf_host.ufilter_md5) i++;
	if (r->uf_global.ufilter_md5) i++;

	/* setup the md5 hash */
	if (i) {
		MD5_Init(&md5);
		MD5_Update(&md5, r->url_host.s, strlen(r->url_host.s));
		MD5_Final(digest, &md5);
		e=r->md5_url;
		for (j=0; j<16; j++) {
			*e = hextab[digest[j]/16]; ++e;
			*e = hextab[digest[j]%16]; ++e;
		}
		*e=0;
		debug(30, 5, "url_init: md5url(%s)", r->md5_url);
	}
	
	i = 0;
	if (r->uf_user.ufilter_sha1) i++;
	if (r->uf_host.ufilter_sha1) i++;
	if (r->uf_global.ufilter_sha1) i++;

	/* setup the sha1 hash */
	if (i) {
		SHA1_Init(&sha1);
		SHA1_Update(&sha1, r->url_host.s, strlen(r->url_host.s));
		SHA1_Final(digest, &sha1);
		e=r->sha1_url;
		for (j=0; j<16; j++) {
			*e = hextab[digest[j]/16]; ++e;
			*e = hextab[digest[j]%16]; ++e;
		}
		*e=0;
		debug(30, 5, "url_init: sha1url(%s)", r->sha1_url);
	}

	return;
}
