
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 14 (host acl checking)
 */

#include "squidwall.h"

/**
 * 
 */
int get_acl_host(request_t *r)
{
	struct squidwall_acl *acl;
	struct list_head *l;
	int found;

	/* initialize the stralloc pointers */
	r->bf_host.ufilter=0;
	r->bf_host.ufilter_url=0;
	r->bf_host.ufilter_dom=0;
	r->bf_host.ufilter_md5=0;
	r->bf_host.ufilter_sha1=0;
	r->bf_host.urfilter=0;

	r->uf_host.ufilter=0;
	r->uf_host.ufilter_url=0;
	r->uf_host.ufilter_dom=0;
	r->uf_host.ufilter_md5=0;
	r->uf_host.ufilter_sha1=0;
	r->uf_host.urfilter=0;

	r->wf_host.ufilter=0;
	r->wf_host.ufilter_url=0;
	r->wf_host.ufilter_dom=0;
	r->wf_host.ufilter_md5=0;
	r->wf_host.ufilter_sha1=0;
	r->wf_host.urfilter=0;

	list_for_each(l, &hosts) {
		acl = list_entry(l, struct squidwall_acl, list);
		found = 0;

		/* check ip */
		if (case_equals(acl->filename.s, r->ip.s)) {
			debug(13,3, "@check_ip(%s)\n", r->ip.s);
			found = 1;
		}

		/* check hostname */
		if (case_equals(acl->filename.s, r->host.s)) {
			debug(13,3, "@check_host(%s)\n", r->host.s);
			found = 1;
		}
		
		if (!found) continue;

		r->bf_host.ufilter      = &acl->bfilter;
		r->bf_host.ufilter_url  = &acl->bfilter_url;
		r->bf_host.ufilter_dom  = &acl->bfilter_dom;
		r->bf_host.ufilter_md5  = &acl->bfilter_md5;
		r->bf_host.ufilter_sha1 = &acl->bfilter_sha1;
		r->bf_host.urfilter     = &acl->brfilter;

		r->uf_host.ufilter      = &acl->wfilter;
		r->uf_host.ufilter_url  = &acl->wfilter_url;
		r->uf_host.ufilter_dom  = &acl->wfilter_dom;
		r->uf_host.ufilter_md5  = &acl->wfilter_md5;
		r->uf_host.ufilter_sha1 = &acl->wfilter_sha1;
		r->uf_host.urfilter     = &acl->wrfilter;

		r->wf_host.ufilter      = &acl->ufilter;
		r->wf_host.ufilter_url  = &acl->ufilter_url;
		r->wf_host.ufilter_dom  = &acl->ufilter_dom;
		r->wf_host.ufilter_md5  = &acl->ufilter_md5;
		r->wf_host.ufilter_sha1 = &acl->ufilter_sha1;
		r->wf_host.urfilter     = &acl->urfilter;

		r->cf_host.pfilter  = &acl->pfilter;
		r->cf_host.cfilter  = &acl->cfilter;
		r->cf_host.crfilter = &acl->crfilter;
		r->cf_host.bcfilter = &acl->bcfilter;

		set(r->todo, acl->todo);
		return acl->flags;
	}

	return ACL_NONE;
}
