
/**
 * Copyright (C) 2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 12 (general acl setup)
 */

#include "squidwall.h"
#include "redir_site.h"

/**
 * get_acls - setup the acl's according to some definitions
 *
 * @param user: current user acl's (defined @ squidwall/users/!)
 * @param host: current host/ip acl's (defined squidwall/hosts/!)
 * @param global: current global acl's (defined @ squidwall/globals/!)
 *
 * @return: ACL_ALLOW  -> the user/host can pass all filters
 *          ACL_FILTER -> filter have to be used
 *          ACL_DENY   -> the user/host will be denied
 */
int get_acls(request_t *r)
{
	int user, host, global;

	user = get_acl_user(r);
	host = get_acl_host(r);
	global = get_acl_global(r);

	if (user == ACL_DENY) {
		redir_site(r, REDIR_TYPE_USER);
		return ACL_DENY;
	}

	if (host == ACL_DENY) {
		redir_site(r, REDIR_TYPE_HOST);
		return ACL_DENY;
	}

	if (user == ACL_FILTER) return ACL_FILTER;
	if (host == ACL_FILTER) return ACL_FILTER;

	if (user == ACL_ALLOW) return ACL_ALLOW;
	if (host == ACL_ALLOW) return ACL_ALLOW;

	/* if user and host are not defined, the global acl's are used */
	if (global == ACL_DENY) {
		redir_site(r, REDIR_TYPE_HOST);
		return ACL_DENY;
	}

	/* return the default global settings, if any */
	return global;
}
