
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 14
 */

#include "squidwall.h"

/* internal functions */
static int check_cont_hdr(request_t *r, array *a, char *line);
static int check_cont_line(request_t *r, char *line);

#define SCAN_UNKNOWN 1
#define SCAN_YES     2
#define SCAN_NO      3
static int check_cont_hdr(request_t *r, array *a, char *line)
{
	struct squidwall_regex *regex;
	struct named_regex *nr;
	int i, j;

	/* last http header was send, no scanning of this content is wanted! */
	if (str_len(line) == 0) {
		debug(14,3, "check_hdr: SCAN_NO @ line(%s)", line);
		return SCAN_NO;
	}

	for (i=0; i < array_length(a, sizeof(struct squidwall_regex)); i++) {
		regex = array_get(a, sizeof(struct squidwall_regex), i);
		for (j=0; j < array_length(&regex->regex, sizeof(struct named_regex)); j++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), j);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				debug(14,3, "check_hdr: SCAN_YES @ line(%s)", line);
				return SCAN_YES;
			}
		}
	}

	debug(14,3, "check_hdr: SCAN_UNKNOWN @ line(%s)", line);

	return SCAN_UNKNOWN;
}

static int check_cont_line(request_t *r, char *line)
{
	struct squidwall_string *string;
	struct squidwall_phrase *phrase;
	struct squidwall_regex *regex;
	struct named_regex *nr;
	struct list_head *l;
	char *file, *str, num[FMT_ULONG];
	unsigned int j, k, k2, *val;

	if (r->cfilter_todo == SCAN_YES)
	if (r->cfilter_user)
	for (j=0; j < r->cfilter_user->len; j += str_len(r->cfilter_user->s+j)+1)
	list_for_each(l, &cfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->cfilter_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(line, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				redir_site(r, REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (r->cfilter_todo == SCAN_YES)
	if (r->cfilter_host)
	for (j=0; j < r->cfilter_host->len; j += str_len(r->cfilter_host->s+j)+1)
	list_for_each(l, &cfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->cfilter_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(line, str)) {
				if (!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if (!stralloc_cats(&r->filter, " / ")) die_nomem();
				if (!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				redir_site(r, REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (r->cfilter_todo == SCAN_YES)
	if (r->cfilter_globals)
	for (j=0; j < r->cfilter_globals->len; j += str_len(r->cfilter_globals->s+j)+1)
	list_for_each(l, &cfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = r->cfilter_globals->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(line, str)) {
				if(!stralloc_copy(&r->filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&r->filter, " / ")) die_nomem();
				if(!stralloc_cats(&r->filter, str)) die_nomem();
				r->filter.s[r->filter.len]=0;
				redir_site(r, REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (r->pfilter_todo == SCAN_YES)
	if (r->pfilter_user)
	for (j=0; j < r->pfilter_user->len; j += str_len(r->pfilter_user->s+j)+1)
	list_for_each(l, &pfilter) {
		phrase = list_entry(l, struct squidwall_phrase, list);
		file = r->pfilter_user->s+j;
		if (str_equal(file, phrase->filename.s))
		for (k=0, k2=0; k < phrase->strings.len;
				k += str_len(phrase->strings.s+k)+1, k2++) {
			str = phrase->strings.s+k;
			val = array_get(&phrase->points, sizeof(unsigned int), k2);
			if (case_strstr(line, str)) {
				num[fmt_ulong(num, *val)]=0;
				if (!stralloc_cat(&r->pfilter_matches, &phrase->filename)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, " (")) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, str)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, " / ")) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, num)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, ")<br>")) die_nomem();
				r->pfilter_current += *val;
				if (r->pfilter_current >= pfilter_max) {
					if (!stralloc_copy(&r->filter, &r->pfilter_matches)) die_nomem();
					redir_site(r, REDIR_TYPE_CONT);
					r->filter.s[r->filter.len]=0;
					return ACL_DENY;
				}
			}
		}
	}

	if (r->pfilter_todo == SCAN_YES)
	if (r->pfilter_host)
	for (j=0; j < r->pfilter_host->len; j += str_len(r->pfilter_host->s+j)+1)
	list_for_each(l, &pfilter) {
		phrase = list_entry(l, struct squidwall_phrase, list);
		file = r->pfilter_host->s+j;
		if (str_equal(file, phrase->filename.s))
		for (k=0, k2=0; k < phrase->strings.len;
				k += str_len(phrase->strings.s+k)+1, k2++) {
			str = phrase->strings.s+k;
			val = array_get(&phrase->points, sizeof(unsigned int), k2);
			if (case_strstr(line, str)) {
				num[fmt_ulong(num, *val)]=0;
				if (!stralloc_cat(&r->pfilter_matches, &phrase->filename)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, " (")) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, str)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, " / ")) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, num)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, ")<br>")) die_nomem();
				r->pfilter_current += *val;
				if (r->pfilter_current >= pfilter_max) {
					if (!stralloc_copy(&r->filter, &r->pfilter_matches)) die_nomem();
					r->filter.s[r->filter.len]=0;
					redir_site(r, REDIR_TYPE_CONT);
					return ACL_DENY;
				}
			}
		}
	}

	if (r->pfilter_todo == SCAN_YES)
	if (r->pfilter_globals)
	for (j=0; j < r->pfilter_globals->len; j += str_len(r->pfilter_globals->s+j)+1)
	list_for_each(l, &pfilter) {
		phrase = list_entry(l, struct squidwall_phrase, list);
		file = r->pfilter_globals->s+j;
		if (str_equal(file, phrase->filename.s))
		for (k=0, k2=0; k < phrase->strings.len;
				k += str_len(phrase->strings.s+k)+1, k2++) {
			str = phrase->strings.s+k;
			val = array_get(&phrase->points, sizeof(unsigned int), k2);
			if (case_strstr(line, str)) {
				num[fmt_ulong(num, *val)]=0;
				if (!stralloc_cat(&r->pfilter_matches, &phrase->filename)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, " (")) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, str)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, " / ")) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, num)) die_nomem();
				if (!stralloc_cats(&r->pfilter_matches, ")<br>")) die_nomem();
				r->pfilter_current += *val;
				if (r->pfilter_current >= pfilter_max) {
					if (!stralloc_copy(&r->filter, &r->pfilter_matches)) die_nomem();
					redir_site(r, REDIR_TYPE_CONT);
					r->filter.s[r->filter.len]=0;
					return ACL_DENY;
				}
			}
		}
	}

	if (r->crfilter_user)
	for (j=0; j < r->crfilter_user->len; j += str_len(r->crfilter_user->s+j)+1)
	list_for_each(l, &crfilter) {
		file = r->crfilter_user->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				if (!stralloc_copy(&r->filter, &regex->filename)) die_nomem();
				if (!stralloc_cats(&r->filter, " / ")) die_nomem();
				if (!stralloc_cat(&r->filter, &nr->n)) die_nomem();
				r->filter.s[r->filter.len]=0;
				redir_site(r, REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (r->crfilter_host)
	for (j=0; j < r->crfilter_host->len; j += str_len(r->crfilter_host->s+j)+1)
	list_for_each(l, &crfilter) {
		file = r->crfilter_host->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				if (!stralloc_copy(&r->filter, &regex->filename)) die_nomem();
				if (!stralloc_cats(&r->filter, " / ")) die_nomem();
				if (!stralloc_cat(&r->filter, &nr->n)) die_nomem();
				r->filter.s[r->filter.len]=0;
				redir_site(r, REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (r->crfilter_globals)
	for (j=0; j < r->crfilter_globals->len; j += str_len(r->crfilter_globals->s+j)+1)
	list_for_each(l, &crfilter) {
		file = r->crfilter_globals->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				if (!stralloc_copy(&r->filter, &regex->filename)) die_nomem();
				if (!stralloc_cats(&r->filter, " / ")) die_nomem();
				if (!stralloc_cat(&r->filter, &nr->n)) die_nomem();
				r->filter.s[r->filter.len]=0;
				redir_site(r, REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (r->bcfilter_user)
	for (j=0; j < r->bcfilter_user->len; j += str_len(r->bcfilter_user->s+j)+1)
	list_for_each(l, &bcfilter) {
		file = r->bcfilter_user->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				redir_site(r, REDIR_TYPE_BANNER);
				return ACL_DENY;
			}
		}
	}

	if (r->bcfilter_host)
	for (j=0; j < r->bcfilter_host->len; j += str_len(r->bcfilter_host->s+j)+1)
	list_for_each(l, &bcfilter) {
		file = r->bcfilter_host->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				redir_site(r, REDIR_TYPE_BANNER);
				return ACL_DENY;
			}
		}
	}

	if (r->bcfilter_globals)
	for (j=0; j < r->bcfilter_globals->len; j += str_len(r->bcfilter_globals->s+j)+1)
	list_for_each(l, &bcfilter) {
		file = r->bcfilter_globals->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				redir_site(r, REDIR_TYPE_BANNER);
				return ACL_DENY;
			}
		}
	}

	return ACL_ALLOW;
}


/**
 * 
 */
int check_cont(request_t *r)
{
	stralloc content_line, content_blob, content_type;
	int len=0, i;
	char ch;

	debug(14,3, "check_cont() todo: %d\n", r->todo);

	if (!str_equal(r->squid_method, "GET"))
		return ACL_ALLOW;

	stralloc_init(&content_line);
	stralloc_init(&content_blob);
	stralloc_init(&content_type);
	stralloc_init(&r->pfilter_matches);

	if (!stralloc_copys(&r->clamav, "")) die_nomem();
	if (!stralloc_copys(&r->filter, "")) die_nomem();
	if (!stralloc_copys(&r->virus, "")) die_nomem();

	/* connect to clamav */
	if (isset(r->todo, TODO_VIRUS)) {
		check_clamav_connect(r);
	}

	/* open connection to proxy */
	r->fd_proxy=socket_tcp4();
	if (r->fd_proxy == -1) {
		panic("socket() for proxy failed");
		return ACL_ALLOW;
	}

        i=socket_connect4(r->fd_proxy, (char*)&proxy_ip, proxy_port);
	if (!socket_connected(r->fd_proxy)) {
		close(r->fd_proxy);
		panic("connect() to proxy failed");
		return ACL_ALLOW;
	}
	if (!io_fd(r->fd_proxy)) {
		panic("io_fd() failed");
		return ACL_ALLOW;
	}

	buffer_init(&r->proxy_in,   (int(*)())read, r->fd_proxy,
		r->proxy_iBUF, sizeof r->proxy_iBUF);
	buffer_init(&r->proxy_out, (int(*)())write, r->fd_proxy,
		r->proxy_oBUF, sizeof r->proxy_oBUF);

	/* send the request to squid */
	io_wantwrite(r->fd_proxy); io_wait(); io_dontwantwrite(r->fd_proxy);
	buffer_puts(&r->proxy_out, r->squid_method);
	buffer_puts(&r->proxy_out, " ");
	buffer_puts(&r->proxy_out, r->squid_url);
	buffer_puts(&r->proxy_out, " HTTP/1.0\r\n");
	buffer_puts(&r->proxy_out, "User-Agent: " VERSION);
	buffer_puts(&r->proxy_out, "\r\n\r\n");
	buffer_flush(&r->proxy_out);

	/* look for bad content (if wanted) */
	r->cfilter_todo = SCAN_UNKNOWN;
	r->pfilter_todo = SCAN_UNKNOWN;
	r->pfilter_current = 0;
	io_wantread(r->fd_proxy); io_wait(); io_dontwantread(r->fd_proxy);
	while (buffer_get(&r->proxy_in, &ch, 1) == 1) {
		if (++len >= maxsize) break;

		/* send it to clamd */
		if (isset(r->todo, TODO_VIRUS)) {
			check_clamav_put2stream(r, ch);
		}

		if (!stralloc_catb(&content_blob, &ch, 1)) die_nomem();
		if (!stralloc_catb(&content_line, &ch, 1)) die_nomem();
		if (ch == '\n') { /* 'HTTP/1.0 200 OK\r\n' */
			if (content_line.len < 2) {
				if (!stralloc_copys(&content_line, "")) die_nomem();
				continue;
			}
			content_line.s[content_line.len-2]=0;

			if (r->cfilter_todo == SCAN_UNKNOWN) {
				r->cfilter_todo = check_cont_hdr(r, &array_cfilter_todo,
					content_line.s);
			}

			if (r->pfilter_todo == SCAN_UNKNOWN) {
				r->pfilter_todo = check_cont_hdr(r, &array_pfilter_todo,
					content_line.s);
			}

			if (check_cont_line(r, content_line.s) == ACL_DENY) {
				/* close proxy stuff */
				r->proxy_out.fd=-1; buffer_close(&r->proxy_out);
				io_close(r->fd_proxy);
				stralloc_free(&content_blob);
				stralloc_free(&content_line);
				stralloc_free(&r->pfilter_matches);
				return ACL_DENY;
			}

			if (content_type.len == 0 &&
			    case_starts(content_line.s, "Content-Type: ")) {
				i = str_chr(content_line.s+14, ';');
				if (i) content_line.s[i+14]=0;
				i = str_chr(content_line.s+14, ' ');
				if (i) content_line.s[i+14]=0;
				if (!stralloc_copys(&content_type, content_line.s+14))
					die_nomem();
			}

			/* delete last content line */
			if (!stralloc_copys(&content_line, "")) die_nomem();
		}
	}

	/* close proxy stuff */
	r->proxy_out.fd=-1; buffer_close(&r->proxy_out); io_close(r->fd_proxy);
	stralloc_free(&content_line);
	stralloc_free(&r->pfilter_matches);

#if WANT_SQLITE
	if (cache) {
		// &content_type, &content_blob
		cache_content(r);
	}
#endif
	stralloc_free(&content_type);
	stralloc_free(&content_blob);

	/* see, what clamav means to our stream */
	if (isset(r->todo, TODO_VIRUS)) {
		if (check_clamav_checkresult(r)) {
			/* log the virus site */
			debug(14,3, "virus: %S\t%S\t%S\t%S\t%S\t%S\t", &r->url,
				&r->ip, &r->host, &r->user, &r->method, &r->virus);
			redir_site(r, REDIR_TYPE_VIRUS);
			stralloc_free(&content_blob);
			stralloc_free(&content_type);
			return ACL_DENY;
		}
	}

	return ACL_ALLOW;
}
