
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 5 (handling of requests and url parsing)
 */

#include "squidwall.h"

/**
 * parse_url - extract host and path information from the squid url
 *
 * needs these global source variable:
 *  char *squid_url   | http://www.foo.org/path/to/bar.jpg
 *
 * fills that two variables:
 *  stralloc url_host | www.foo.org
 *  stralloc url_path | /path/to/bar.jpg
 *
 * @return nothing
 */
void parse_url(request_t *r)
{
	stralloc sa;
	char *x;

	x=r->squid_url;
	while (*x++) {
		if (str_start(x, "://"))   { x+=3; break; }
	}

	/* there is no "://" in the url */
	if (!*x) x=r->squid_url;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, x)) die_nomem();
	if (!stralloc_0(&sa)) die_nomem();

	/* www.domainname.de/xyz -> xyz isn't wanted (md5, sha1 lists!) */
	x = sa.s;
	while (*x++) {
		if (*x == '/') {
			*x=0;
			if (!stralloc_copys(&r->url_host, sa.s)) die_nomem();
			r->url_host.s[r->url_host.len]=0;
			if (x+1) {
				if (!stralloc_copys(&r->url_path, x+1)) die_nomem();
				r->url_path.s[r->url_path.len]=0;
			} else {
				if (!stralloc_copys(&r->url_path, "")) die_nomem();
				r->url_path.s[r->url_path.len]=0;
			}
			stralloc_free(&sa);
			return;
		}
	}
	if (!stralloc_copys(&r->url_host, sa.s)) die_nomem();
	if (!stralloc_copys(&r->url_path, "/")) die_nomem();
	r->url_host.s[r->url_host.len]=0;
	r->url_path.s[r->url_path.len]=0;
	stralloc_free(&sa);

	return;
}
