
/**
 * Copyright (C) 2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 2 (reading controls)
 */

#include "squidwall.h"
#include "scanX.h"

/* configuration variables */
static stralloc sa_proxy;
static stralloc sa_logfile;
static stralloc sa_host_dir;
static stralloc sa_user_dir;
static stralloc sa_cache_dir;
static stralloc sa_redir_dir;
static stralloc sa_globals_dir;
static stralloc sa_clamav_tcp;
static stralloc sa_clamav_socket;

char *proxy=PROXY;
char *logfile=LOGFILE;
char *host_dir=USER_DIR;
char *user_dir=USER_DIR;
char *cache_dir=CACHE_DIR;
char *redir_dir=REDIR_DIR;
char *globals_dir=GLOBAL_DIR;
char *clamav_tcp=CLAMAV_TCP;
char *clamav_socket=CLAMAV_SOCKET;

int cache=0;
int maxsize=MAXSIZE;
int memcache=MEMCACHE;
int pfilter_max=PFILTER_MAX;
int pfilter_default=PFILTER_DEFAULT;
int sqlite_timeout=SQLITE_TIMEOUT;

struct ip_address clamd_ip;
struct ip_address proxy_ip;

unsigned clamd_port;
unsigned clams_port;
unsigned proxy_port;

/* destination websites for squid */
static stralloc sa_redir_badcont;
static stralloc sa_redir_badhost;
static stralloc sa_redir_badurl;
static stralloc sa_redir_baduser;
static stralloc sa_redir_bannerfound;
static stralloc sa_redir_safesite;
static stralloc sa_redir_virusfound;
static stralloc sa_redir_webinterface;

char *redir_badurl=REDIR_BADURL;
char *redir_badcont=REDIR_BADCONT;
char *redir_baduser=REDIR_BADUSER;
char *redir_badhost=REDIR_BADHOST;
char *redir_safesite=REDIR_SAFESITE;
char *redir_virusfound=REDIR_VIRUSFOUND;
char *redir_bannerfound=REDIR_BANNERFOUND;
char *redir_webinterface=REDIR_WEBINTERFACE;

/* filters */
static stralloc sa_bfilter_dir;
static stralloc sa_brfilter_dir;
static stralloc sa_bfilter_url_dir;
static stralloc sa_bfilter_dom_dir;
static stralloc sa_bfilter_md5_dir;
static stralloc sa_bfilter_sha1_dir;

static stralloc sa_ufilter_dir;
static stralloc sa_urfilter_dir;
static stralloc sa_ufilter_url_dir;
static stralloc sa_ufilter_dom_dir;
static stralloc sa_ufilter_md5_dir;
static stralloc sa_ufilter_sha1_dir;

static stralloc sa_wfilter_dir;
static stralloc sa_wrfilter_dir;
static stralloc sa_wfilter_url_dir;
static stralloc sa_wfilter_dom_dir;
static stralloc sa_wfilter_md5_dir;
static stralloc sa_wfilter_sha1_dir;

static stralloc sa_pfilter_dir;
static stralloc sa_cfilter_dir;
static stralloc sa_crfilter_dir;
static stralloc sa_bcfilter_dir;

char *bfilter_dir=BFILTER_DIR;
char *brfilter_dir=BRFILTER_DIR;
char *bfilter_url_dir=BFILTER_URL_DIR;
char *bfilter_dom_dir=BFILTER_DOM_DIR;
char *bfilter_md5_dir=BFILTER_MD5_DIR;
char *bfilter_sha1_dir=BFILTER_SHA1_DIR;

char *ufilter_dir=UFILTER_DIR;
char *urfilter_dir=URFILTER_DIR;
char *ufilter_url_dir=UFILTER_URL_DIR;
char *ufilter_dom_dir=UFILTER_DOM_DIR;
char *ufilter_md5_dir=UFILTER_MD5_DIR;
char *ufilter_sha1_dir=UFILTER_SHA1_DIR;

char *wfilter_dir=WFILTER_DIR;
char *wrfilter_dir=WRFILTER_DIR;
char *wfilter_url_dir=WFILTER_URL_DIR;
char *wfilter_dom_dir=WFILTER_DOM_DIR;
char *wfilter_md5_dir=WFILTER_MD5_DIR;
char *wfilter_sha1_dir=WFILTER_SHA1_DIR;

char *pfilter_dir=PFILTER_DIR;
char *cfilter_dir=CFILTER_DIR;
char *crfilter_dir=CRFILTER_DIR;
char *bcfilter_dir=BCFILTER_DIR;

/**
 * level: 1|2|4|8|16|32|64|128|256 => 1..9
 * section: 1..MAX_SECTION
 */
unsigned int debugging[MAX_SECTION];

static void read_controls_init(void);
static int read_controls_int(char *path, char *file, int *oldval);
static int read_controls_str(char *path, char *file, stralloc *val, char *old);
static int read_controls_dbg(char *path, char *file);

/* threadsafe - should be: because of read_acls_mutex ... */
static time_t t_controls;
static int done=0;

static void read_controls_init(void)
{
	if (done) return;
	done=1;

	stralloc_init(&sa_proxy);
	stralloc_init(&sa_logfile);
	stralloc_init(&sa_host_dir);
	stralloc_init(&sa_user_dir);
	stralloc_init(&sa_cache_dir);
	stralloc_init(&sa_redir_dir);
	stralloc_init(&sa_globals_dir);
	stralloc_init(&sa_clamav_tcp);
	stralloc_init(&sa_clamav_socket);

	stralloc_init(&sa_redir_badcont);
	stralloc_init(&sa_redir_badhost);
	stralloc_init(&sa_redir_badurl);
	stralloc_init(&sa_redir_baduser);
	stralloc_init(&sa_redir_bannerfound);
	stralloc_init(&sa_redir_safesite);
	stralloc_init(&sa_redir_virusfound);
	stralloc_init(&sa_redir_webinterface);

	stralloc_init(&sa_bfilter_dir);
	stralloc_init(&sa_brfilter_dir);
	stralloc_init(&sa_bfilter_url_dir);
	stralloc_init(&sa_bfilter_dom_dir);
	stralloc_init(&sa_bfilter_md5_dir);
	stralloc_init(&sa_bfilter_sha1_dir);

	stralloc_init(&sa_ufilter_dir);
	stralloc_init(&sa_urfilter_dir);
	stralloc_init(&sa_ufilter_url_dir);
	stralloc_init(&sa_ufilter_dom_dir);
	stralloc_init(&sa_ufilter_md5_dir);
	stralloc_init(&sa_ufilter_sha1_dir);

	stralloc_init(&sa_wfilter_dir);
	stralloc_init(&sa_wrfilter_dir);
	stralloc_init(&sa_wfilter_url_dir);
	stralloc_init(&sa_wfilter_dom_dir);
	stralloc_init(&sa_wfilter_md5_dir);
	stralloc_init(&sa_wfilter_sha1_dir);

	stralloc_init(&sa_pfilter_dir);
	stralloc_init(&sa_cfilter_dir);
	stralloc_init(&sa_crfilter_dir);
	stralloc_init(&sa_bcfilter_dir);

	return;
}


static int read_controls_int(char *path, char *file, int *old)
{
	stralloc sa;
	int rv;

	stralloc_init(&sa);

	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/")) die_nomem();
	if (!stralloc_cats(&sa, file)) die_nomem();
	sa.s[sa.len]=0;

	rv = control_readint(sa.s, old);
	stralloc_free(&sa);

	if (rv == 1) return 1;

	return 0;
}

static int read_controls_str(char *path, char *file, stralloc *val, char *old)
{
	stralloc sa;
	int rv;

	/* initialize all control stralloc's */
	read_controls_init();

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/")) die_nomem();
	if (!stralloc_cats(&sa, file)) die_nomem();
	sa.s[sa.len]=0;

	rv = control_readline(sa.s, val, old);
	stralloc_free(&sa);

	if (rv == 1) return 1;

	return 0;
}

static int read_controls_dbg(char *path, char *file)
{
	stralloc sa, val;
	unsigned int level, section, i;
	unsigned int old[MAX_SECTION];
	int rv;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/")) die_nomem();
	if (!stralloc_cats(&sa, file)) die_nomem();
	sa.s[sa.len]=0;

	/* get old values - for comparing later */
	for (i=0; i<MAX_SECTION; i++) {
		old[i] = debugging[i];
	}

	stralloc_init(&val);
	control_readline(sa.s, &val, "");
	val.s[val.len]=0;

	/**
	 * 1,3 5,4 6,3   9,4 ...
	 * -> default: x,3
	 */
	for (i=0; i<val.len;) {
		rv = scan_dbg(val.s+i, &section, &level);
		if (rv) {
			/* we have an entry */
			if (section < MAX_SECTION) {
				debugging[section]=level;
			}
			i+=rv;
		} else i++;
	}

	rv = 0;
	for (i=0; i<MAX_SECTION; i++) {
		/* default value for unset ones */
		if (debugging[i] == 0) { debugging[i]=3; rv = 1; }
		if (old[i] != debugging[i]) { rv = 1; }
		debug(2,9, "debugging old[%2u]=%2u", i, old[i]);
		debug(2,9, "debugging new[%2u]=%2u", i, debugging[i]);
	}

	debug(2,8, "returning %u ...", rv);

	stralloc_free(&sa);
	stralloc_free(&val);

	return rv;
}

/**
 * read_controls - re-read squidwall variables
 */
int read_controls(request_t *r)
{
	int rv, changes=0;
	struct stat s;
	stralloc sa;
	char *path=CONTROL_DIR;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/!")) die_nomem();
	if (!stralloc_0(&sa)) die_nomem();

	/* if stat fails, we return 0 (meaning no change) */
	if (stat(sa.s, &s) == -1) {
		stralloc_free(&sa);
		return 0;
	}
	stralloc_free(&sa);

	if (t_controls == s.st_mtime) {
		return 0; /* was not modified... */
	}

	/* was modified... */
	t_controls = s.st_mtime;

	rv = read_controls_dbg(path, "debug");
	if (rv) { changes++; }

	rv = read_controls_int(path, "cache", &cache);
	if (rv) changes++;

	rv = read_controls_int(path, "maxsize", &maxsize);
	if (rv) changes++;

	rv = read_controls_int(path, "memcache", &memcache);
	if (rv) changes++;

	rv = read_controls_int(path, "pfilter_max", &pfilter_max);
	if (rv) changes++;

	rv = read_controls_int(path, "pfilter_default", &pfilter_default);
	if (rv) changes++;

	rv = read_controls_int(path, "sqlite_timeout", &sqlite_timeout);
	if (rv) changes++;

	rv = read_controls_str(path, "proxy", &sa_proxy, proxy);
	if (rv) { changes++; proxy=sa_proxy.s; }
	rv = ip4_scan(proxy, (char*)&proxy_ip);
	if (!rv) panic("can't get ip of proxy!");
	rv = scan_uint(proxy+rv+1, &proxy_port);
	if (!rv) panic("can't get port of proxy!");

	rv = read_controls_str(path, "logfile",
		&sa_logfile, logfile);
	if (rv) { changes++; logfile=sa_logfile.s; log_init(r,1); }


	rv = read_controls_str(path, "host_dir",
		&sa_host_dir, host_dir);
	if (rv) { changes++; host_dir=sa_host_dir.s; }

	rv = read_controls_str(path, "user_dir",
		&sa_user_dir, user_dir);
	if (rv) { changes++; user_dir=sa_user_dir.s; }

	rv = read_controls_str(path, "cache_dir",
		&sa_cache_dir, cache_dir);
	if (rv) { changes++; cache_dir=sa_cache_dir.s; }

	rv = read_controls_str(path, "redir_dir",
		&sa_redir_dir, redir_dir);
	if (rv) { changes++; redir_dir=sa_redir_dir.s; }

	rv = read_controls_str(path, "globals_dir",
		&sa_globals_dir, globals_dir);
	if (rv) { changes++; globals_dir=sa_globals_dir.s; }


	/**
	 * clamav
	 */

	rv = read_controls_str(path, "clamav_tcp",
		&sa_clamav_tcp, clamav_tcp);
	if (rv) { changes++; clamav_tcp=sa_clamav_tcp.s; }
	rv = ip4_scan(clamav_tcp, (char*)&clamd_ip);
	if (!rv) panic("can't get ip of clamav daemon!");
	rv = scan_uint(clamav_tcp+rv+1, &clamd_port);
	if (!rv) panic("can't get port of clamav daemon!");

	rv = read_controls_str(path, "clamav_socket",
		&sa_clamav_socket, clamav_socket);
	if (rv) { changes++; clamav_socket=sa_clamav_socket.s; }


	/**
	 * redirection
	 */

	rv = read_controls_str(path, "redir_badcont",
		&sa_redir_badcont, redir_badcont);
	if (rv) { changes++; redir_badcont=sa_redir_badcont.s; }

	rv = read_controls_str(path, "redir_badhost",
		&sa_redir_badhost, redir_badhost);
	if (rv) { changes++; redir_badhost=sa_redir_badhost.s; }

	rv = read_controls_str(path, "redir_badurl",
		&sa_redir_badurl, redir_badurl);
	if (rv) { changes++; redir_badurl=sa_redir_badurl.s; }

	rv = read_controls_str(path, "redir_baduser",
		&sa_redir_baduser, redir_baduser);
	if (rv) { changes++; redir_baduser=sa_redir_baduser.s; }

	rv = read_controls_str(path, "redir_bannerfound",
		&sa_redir_bannerfound, redir_bannerfound);
	if (rv) { changes++; redir_bannerfound=sa_redir_bannerfound.s; }

	rv = read_controls_str(path, "redir_safesite",
		&sa_redir_safesite, redir_safesite);
	if (rv) { changes++; redir_safesite=sa_redir_safesite.s; }

	rv = read_controls_str(path, "redir_virusfound"
		, &sa_redir_virusfound, redir_virusfound);
	if (rv) { changes++; redir_virusfound=sa_redir_virusfound.s; }

	rv = read_controls_str(path, "redir_webinterface",
		&sa_redir_webinterface, redir_webinterface);
	if (rv) { changes++; redir_webinterface=sa_redir_webinterface.s; }


	/**
	 * url filter - banners
	 */

	rv = read_controls_str(path, "bfilter_dir",
		&sa_bfilter_dir, bfilter_dir);
	if (rv) { changes++; bfilter_dir=sa_bfilter_dir.s; }

	rv = read_controls_str(path, "brfilter_dir",
		&sa_brfilter_dir, brfilter_dir);
	if (rv) { changes++; brfilter_dir=sa_brfilter_dir.s; }

	rv = read_controls_str(path, "bfilter_url_dir",
		&sa_bfilter_url_dir, bfilter_url_dir);
	if (rv) { changes++; bfilter_url_dir=sa_bfilter_url_dir.s; }

	rv = read_controls_str(path, "bfilter_dom_dir",
		&sa_bfilter_dom_dir, bfilter_dom_dir);
	if (rv) { changes++; bfilter_dom_dir=sa_bfilter_dom_dir.s; }

	rv = read_controls_str(path, "bfilter_md5_dir",
		&sa_bfilter_md5_dir, bfilter_md5_dir);
	if (rv) { changes++; bfilter_md5_dir=sa_bfilter_md5_dir.s; }

	rv = read_controls_str(path, "bfilter_sha1_dir",
		&sa_bfilter_sha1_dir, bfilter_sha1_dir);
	if (rv) { changes++; bfilter_sha1_dir=sa_bfilter_sha1_dir.s; }


	/**
	 * url filter - blacklists
	 */

	rv = read_controls_str(path, "ufilter_dir",
		&sa_ufilter_dir, ufilter_dir);
	if (rv) { changes++; ufilter_dir=sa_ufilter_dir.s; }

	rv = read_controls_str(path, "urfilter_dir",
		&sa_urfilter_dir, urfilter_dir);
	if (rv) { changes++; urfilter_dir=sa_urfilter_dir.s; }

	rv = read_controls_str(path, "ufilter_url_dir",
		&sa_ufilter_url_dir, ufilter_url_dir);
	if (rv) { changes++; ufilter_url_dir=sa_ufilter_url_dir.s; }

	rv = read_controls_str(path, "ufilter_dom_dir",
		&sa_ufilter_dom_dir, ufilter_dom_dir);
	if (rv) { changes++; ufilter_dom_dir=sa_ufilter_dom_dir.s; }

	rv = read_controls_str(path, "ufilter_md5_dir",
		&sa_ufilter_md5_dir, ufilter_md5_dir);
	if (rv) { changes++; ufilter_md5_dir=sa_ufilter_md5_dir.s; }

	rv = read_controls_str(path, "ufilter_sha1_dir",
		&sa_ufilter_sha1_dir, ufilter_sha1_dir);
	if (rv) { changes++; ufilter_sha1_dir=sa_ufilter_sha1_dir.s; }


	/**
	 * url filter - whitelists
	 */

	rv = read_controls_str(path, "wfilter_dir",
		&sa_wfilter_dir, wfilter_dir);
	if (rv) { changes++; wfilter_dir=sa_wfilter_dir.s; }

	rv = read_controls_str(path, "wrfilter_dir",
		&sa_wrfilter_dir, wrfilter_dir);
	if (rv) { changes++; wrfilter_dir=sa_wrfilter_dir.s; }

	rv = read_controls_str(path, "wfilter_url_dir",
		&sa_wfilter_url_dir, wfilter_url_dir);
	if (rv) { changes++; wfilter_url_dir=sa_wfilter_url_dir.s; }

	rv = read_controls_str(path, "wfilter_dom_dir",
		&sa_wfilter_dom_dir, wfilter_dom_dir);
	if (rv) { changes++; wfilter_dom_dir=sa_wfilter_dom_dir.s; }

	rv = read_controls_str(path, "wfilter_md5_dir",
		&sa_wfilter_md5_dir, wfilter_md5_dir);
	if (rv) { changes++; wfilter_md5_dir=sa_wfilter_md5_dir.s; }

	rv = read_controls_str(path, "wfilter_sha1_dir",
		&sa_wfilter_sha1_dir, wfilter_sha1_dir);
	if (rv) { changes++; wfilter_sha1_dir=sa_wfilter_sha1_dir.s; }


	/**
	 * content filter
	 */

	rv = read_controls_str(path, "pfilter_dir",
		&sa_pfilter_dir, pfilter_dir);
	if (rv) { changes++; pfilter_dir=sa_pfilter_dir.s; }
	rv = read_controls_str(path, "cfilter_dir",
		&sa_cfilter_dir, cfilter_dir);
	if (rv) { changes++; cfilter_dir=sa_cfilter_dir.s; }

	rv = read_controls_str(path, "crfilter_dir",
		&sa_crfilter_dir, crfilter_dir);
	if (rv) { changes++; crfilter_dir=sa_crfilter_dir.s; }

	rv = read_controls_str(path, "bcfilter_dir",
		&sa_bcfilter_dir, bcfilter_dir);
	if (rv) { changes++; bcfilter_dir=sa_bcfilter_dir.s; }

	return changes;
}
