#ifndef SW_REQUEST_H
#define SW_REQUEST_H

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* squidwall */
#include "params.h"

/* libowfat */
#include "buffer.h"
#include "io.h"
#include "stralloc.h"

typedef struct {
	stralloc *ufilter;
	stralloc *ufilter_url;
	stralloc *ufilter_dom;
	stralloc *ufilter_md5;
	stralloc *ufilter_sha1;
	stralloc *urfilter;
} url_filter_t;

typedef struct {
	stralloc *pfilter;
	stralloc *cfilter;
	stralloc *crfilter;
	stralloc *bcfilter;
} cnt_filter_t;

typedef struct {

	/* current number in squid request */
	unsigned int tid;

	/* current worker, which handles this request */
	unsigned int wid;

	/* what we have to do for this request */
	int todo;

	/* pointer to the infos from squid @ fd=0 */
	char *squid_ip;
	char *squid_url;
	char *squid_host;
	char *squid_user;
	char *squid_method;

	/* decoded infos */
	stralloc ip;
	stralloc url;
	stralloc host;
	stralloc user;
	stralloc method;

	/* decoded variables for the destination site */
	stralloc redirsite;
	stralloc filter;
	stralloc clamav;
	stralloc virus;

	/* splitted squid_url for use with md5/sha1 lists in check_blist() */
	stralloc url_host; /* -> www.example.org */
	stralloc url_path; /* -> /some/path/ */
	char md5_url[33];  /* -> md5(url_host) */
	char sha1_url[41]; /* -> sha1(url_host) */

	/* url filters (3x 3x6filter) */
	url_filter_t bf_host;
	url_filter_t uf_host;
	url_filter_t wf_host;

	url_filter_t bf_user;
	url_filter_t uf_user;
	url_filter_t wf_user;

	url_filter_t bf_global;
	url_filter_t uf_global;
	url_filter_t wf_global;

	/* content filters (3x4filter) */
	cnt_filter_t cf_host;
	cnt_filter_t cf_user;
	cnt_filter_t cf_global;

	/* these filtering has to be done - we need another list, to be sure
	 * that the same things are not verified two times...
	 */
	url_filter_t bf_todo;
	url_filter_t uf_todo;
	url_filter_t wf_todo;
	cnt_filter_t cf_todo;

	/* ??? */
	int cfilter_todo;
	int pfilter_todo;
	int pfilter_current;
	stralloc pfilter_matches;

	/* buffers for proxy connection */
	int64 fd_proxy;
	char proxy_oBUF[BUFSIZE]; buffer proxy_out;
	char proxy_iBUF[BUFSIZE]; buffer proxy_in;

	/* command and stream connection to clamd */
	int64 fd_clamd, fd_stream;
	char clamd_iBUF[BUFSIZE]; buffer clamd_in;
	char clamd_oBUF[BUFSIZE]; buffer clamd_out;
	char clams_oBUF[BUFSIZE]; buffer clams_out;

	/* buffers for logging */
	char log_BUF[BUFSIZE];
	buffer logbuf;

	/* for sqlite3 caching */
#if WANT_SQLITE
	stralloc content_type;
	stralloc content_blob;
#endif
} request_t;

/* handle_request.c */
extern void handle_request(request_t *r);

#endif /* SW_HANDLE_REQUEST_H */
