
/**
 * Copyright (C) 2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 6 (threading)
 */

/* squidwall */
#include "squidwall.h"
#include "thread.h"
#include "request.h"

#if !THREADPOOL

/**
 * thread management - without a threadpool
 *
 * - we create for each request one thread
 * - the thread will be destroyed, if the work is done
 * - on linux a clone() seems very cheap ... so I leave it for now
 * - I don't know how it performs on *BSD, Solaris ... please tell me!
 */
static void *thread_worker(void *r);

/**
 * thread_init - initialize the threading
 *
 * - there is no need to do sth. here ;)
 */
void thread_init(void)
{
	return;
}

/**
 * thread_operation - create a thread and put some work to it
 *
 * @param fn function to execute
 * @param arg argument to the function
 * @return
 *   0 = everything was okay
 *  -1 = some error detected
 */
int thread_operation(void (*fn)(request_t *r))
{
	pthread_t p;
	int rv;

	/* no function given */
	if (fn == NULL) return -1;

	rv = pthread_create(&p, 0, thread_worker, (void*)fn);
	if (rv) die_pthread();

	rv = pthread_join(p, 0);
	if (rv) die_pthread();

	return 0;
}


/**
 * thread_worker - do some work for thread_operation()
 *
 * @param fn the function, which will be executed
 */
static void *thread_worker(void *fn)
{
	struct squidwall_work *w;
	request_t *r;

	r = (request_t *)malloc(sizeof(request_t));
	w = (struct squidwall_work *)malloc(sizeof(struct squidwall_work));
	log_init(r,0);

	w->fn = fn;
	(*w->fn)(r);
	
	free(r);
	free(w);

	return fn;
}

#endif
